import os
from constants import *


list_of_images=sorted([f for f in os.listdir(FOLDER_ART) if f[-3:] in ['png','jpg']])

cache={}

arts={}
for file_name in ('obj_fox.png','obj_dog_brown.png','obj_dog_grey.png','obj_dog_dalmatian.png','obj_horse_blue.png','obj_horse_brown.png','obj_horse_green.png','obj_horse_red.png',):
	arts[file_name]={}
	arts[file_name]['rows']=9
	arts[file_name]['columns']=4
	arts[file_name]['default']=range(1,5)
	dirs=('r','l','d','u','ru','lu','ld','rd')
	for i in range(len(dirs)):
		arts[file_name][dirs[i]]=range(i*4+1,(i+1)*4+1)
	arts[file_name]['dead']=range(33,37)


file_name='obj_tree.png'
arts[file_name]={}
arts[file_name]['rows']=3
arts[file_name]['columns']=3
arts[file_name]['default']=[1]
for i in range(1,10):
	arts[file_name][str(i)]=[i]

for file_name in ('obj_bush.png','obj_hurdle.png','map_fox.png','map_farm.png'):
	arts[file_name]={}
	arts[file_name]['rows']=1
	arts[file_name]['columns']=1
	arts[file_name]['default']=[1]
	arts[file_name]['1']=[1]

file_name='obj_grass.png'
arts[file_name]={}
arts[file_name]['rows']=4
arts[file_name]['columns']=4
arts[file_name]['default']=[1]
for i in range(1,17):
	arts[file_name][str(i)]=[i]

file_name='obj_trap.png'
arts[file_name]={}
arts[file_name]['rows']=1
arts[file_name]['columns']=3
arts[file_name]['default']=[1]
arts[file_name]['1']=[1]
arts[file_name]['hit']=[2,2,2,3]
arts[file_name]['death']=[3]


file_name='obj_chicken.png'
arts[file_name]={}
arts[file_name]['rows']=9
arts[file_name]['columns']=2
arts[file_name]['default']=[1,2]
dirs=('r','l','d','u','ru','lu','ld','rd')
for i in range(len(dirs)):
	arts[file_name][dirs[i]]=range(i*2+1,(i+1)*2+1)
arts[file_name]['death']=[17,18]
	

def load_next_image():
	img=list_of_images.pop()
	print('Loading', img)
	cache[img]=pygame.image.load(os.path.join(FOLDER_ART, img)).convert_alpha()
	if img[:6]=='screen':
		cache[img]=pygame.transform.scale(cache[img],DISPLAY).convert_alpha()


class Art():
	'''
	Object storing an image and its frame-split version.
	Frames are assigned to actions listed in the art dictionary:
	- dictionary key is the name of the animation
	- dictionary value is a list of images (pygame.Surface)
	Default frame is defined as first frame of image.
	'''
	def __init__(self, name, size):
		self.name=name
		self.image=cache[name]
		self.size=size
		
		self.frame={}		
		if name in arts.keys():
			self.split_image(arts[name])
		else:
			self.frame['default']=[pygame.transform.scale(self.image).convert_alpha(),self.size]
			
		self.curr_action='default'
		self.next_action='default'
		self.curr_index=0
		self.repeat=True
		self.stopped=True
		self.jumping=False

	def split_image(self, details):
		frame_width = self.image.get_width()/details['columns']
		frame_height = self.image.get_height()/details['rows']
		frames=[]
		for row in range(details['rows']):
			for col in range(details['columns']):
				singleFrame=pygame.transform.scale(self.image.subsurface((col*frame_width,row*frame_height,frame_width,frame_height)),self.size).convert_alpha()	
				frames.append(singleFrame)
		
		
		for action in [a for a in details.keys() if a not in ('rows','columns')]:
			self.frame[action]=[]
			for frame_index in details[action]:
				self.frame[action].append(frames[frame_index-1])
	
	def get_image(self):
		return self.image
		
	def set(self, action, immediate=True, repeat=True, stopped=False, index=0):
		if immediate:
			self.curr_action=action
			self.index=index
			self.repeat=repeat
			self.stopped=stopped
			
	def stop(self):
		self.stopped=True
		
	def jump(self, jump):
		self.jumping=jump
	
	def next(self):
		if self.stopped:
			return self.frame[self.curr_action][0]
		elif self.jumping:
			return self.frame[self.curr_action][-1]
		self.curr_index=(self.curr_index+1)%len(self.frame[self.curr_action])
		if not self.repeat and self.curr_index==0 and self.next_action<>None:
			self.curr_action=self.next_action
		return self.frame[self.curr_action][self.curr_index]
			
	