import os, sys, random, math, pygame, math
import tools, art
from constants import *
import objects


class LevelManager(object):
	def __init__(self):
		self.go_to(Splash())

	def go_to(self, next):
		self.level = next
		self.level.clock=0
		self.level.manager = self
		self.level.set_background()
		
		
class Level(object):
	def __init__(self):
		self.sprites=pygame.sprite.LayeredDirty()
		
	def set_background(self):
		try:
			self.background=art.cache[self.bg_file]
		except (KeyError, AttributeError):
			self.background=pygame.Surface(DISPLAY)
			self.background.fill([0,255,0])
			
	
	def addobj(self, obj):
		'''Adds an object to the LayeredDirty group of sprites
		The layer is retieved from the position of the name of the object class (or any parent class) in LAYERLIST, default layer is used otherwise.
		'''
		#classAndParents=[obj.__class__.__name__]+[parent.__name__ for parent in obj.__class__.__bases__]
		#layerNum=[LAYERLIST.index(name) for name in classAndParents if name in LAYERLIST][:1]
		layerNum=obj.rect.centery
		self.sprites.add(obj, layer=layerNum)
		return obj
		
	def remobj(self, obj):
		'''Deletes an object from the LayeredDirty group of sprites
		'''	
		if obj in self.sprites:
			self.sprites.remove(obj)
		else:
			raise NotImplementedError(self.__class__)

	def render(self, screen):
		raise NotImplementedError(self.__class__)

	def update(self):
		raise NotImplementedError(self.__class__)

	def handle_events(self, events):
		raise NotImplementedError(self.__class__)

		



class Splash(Level):
	def __init__(self):
		Level.__init__(self)
		self.bg_file='screen_forkedtaillogo.png'
	
	def set_background(self):
		self.background=tools.load_image(FOLDER_ART, self.bg_file)

	def update(self):
		'''cache all the image files placed in art folder'''
		try:
			art.load_next_image()
		except IndexError:
			self.manager.go_to(Intro(1))
			#self.manager.go_to(FoxLevel(0))
			#self.manager.go_to(Credits())
			
	
	def handle_events(self, events):
		'''game continues when all the images are loaded'''
		for e in events:
			if e.type == pygame.KEYDOWN:
				if e.key==pygame.K_ESCAPE:
					sys.exit('ESC key pressed')
				elif len(art.list_of_images)==0:
					self.manager.go_to(Intro(1))
					#self.manager.go_to(FoxLevel(0))
					#self.manager.go_to(Credits())
	
class Intro(Level):
	def __init__(self, intro):
		self.bg_file_prefix='screen_story.png'
		self.intro=intro
		self.bg_file=self.get_next_bg()
		Level.__init__(self)
		self.timer=0


		
	def get_next_bg(self):
		tmp=self.bg_file_prefix.split('.')
		tmp.insert(1,str(self.intro)+'.')
		return ''.join(tmp)

	def update(self):
		self.timer+=1
		if self.timer>=FPS:
			pass
			

	def handle_events(self,events):
		for e in events:
			if e.type == pygame.KEYDOWN:
				if e.key==pygame.K_ESCAPE:
					sys.exit('ESC key pressed')
				else:
					if self.intro<INTRO:
						self.manager.go_to(Intro(self.intro+1))
					else:
						self.manager.go_to(Map(0))

class Credits(Level):
	def __init__(self):
		self.bg_file='screen_credits.png'	
		Level.__init__(self)
	
	def update(self):
		pass
	
	def handle_events(self,events):
		for e in events:
			if e.type == pygame.KEYDOWN:
				sys.exit('Thanks for playing')


class Map(Level):
	def __init__(self, level):
		Level.__init__(self)
		self.level=level
		self.bg_file='screen_title.png'
		start_position=[[0,0],[1.5*CELLSIZE,4*CELLSIZE],[4.5*CELLSIZE,5*CELLSIZE],[6*CELLSIZE,7*CELLSIZE],[9*CELLSIZE,5*CELLSIZE]]
		#for i in range(self.level):
		#	self.addobj(objects.Farm(start_position[i+1]))
		self.fox=self.addobj(objects.Map_Fox(start_position[level:level+2]))
		if not MUSIC.get_busy():
			MUSIC.play(-1)	
			pass

	def update(self):
		pass

	def handle_events(self, events):
		for e in events:
			if e.type == pygame.KEYDOWN:
				if e.key==pygame.K_ESCAPE:
					sys.exit('ESC key pressed')
				elif e.key in (pygame.K_1,pygame.K_2,pygame.K_3,pygame.K_4):
					self.manager.go_to(Map([pygame.K_1,pygame.K_2,pygame.K_3,pygame.K_4].index(e.key)))
				else:
					if self.fox.ready:
						self.manager.go_to(FoxLevel(self.level))




						
class FoxLevel(Level):
	def __init__(self, level):
		self.bg_file='screen_background.png'
		Level.__init__(self)
		self.level=level
		self.dirkeys=[False]*4
		self.lastkeys=[False]*4
		self.keypresstime=[0.0]*4
		self.speed=0
		self.maxspeed=MAXSPEED
		self.speed_decrease=0

		self.scroll_index=0
		self.scroll_total=0
		self.direction='rdlu'
		self.status='intro'
		self.timer=0
		self.dogs=[]
		self.horses=[]
		self.stuff=[]
		self.change_status('ingame')
		if not MUSIC.get_busy():
			MUSIC.play(-1)			

		self.points=0
		
	def start_map(self):
		for s in self.sprites:
			self.remobj(s)

			
		if self.status=='ingame':
			[self.addobj(objects.Bush([i*CELLSIZE,CELLSIZE])) for i in range(CELL_HNUM+1)]
			
			self.fox=self.addobj(objects.Fox(FOX_POS))	
			
			#add dogs at level
			for i in range(4*(self.level==0)+6*(self.level==1)+10*(self.level==2)+20*(self.level==3)):
				self.dogs.append(self.addobj(objects.Dog([CELLSIZE*random.randrange(-CELL_VNUM/2,0),CELLSIZE*random.randrange(2,CELL_VNUM-2)])))		
				#self.dogs.append(self.addobj(objects.Dog([CELLSIZE*random.randrange(-CELL_VNUM,0),CELLSIZE*CELL_VNUM/2])))	

			
			#add horses at level
			for i in range(1*(self.level==0)+3*(self.level==1)+6*(self.level==2)+12*(self.level==3)):
				self.horses.append(self.addobj(objects.Horse([CELLSIZE*random.randrange(-CELL_VNUM,-CELL_VNUM/2),CELLSIZE*random.randrange(2,CELL_VNUM-2)])))		
			
			[self.addobj(objects.Bush([i*CELLSIZE,(CELL_VNUM-1)*CELLSIZE])) for i in range(CELL_HNUM+1)]



		

	def change_status(self, status):
		self.status=status
		self.start_map()
		
	def update(self):
		self.timer+=1
		if self.status=='ingame':
			self.update_ingame()
			#self.check_deaths()
		
		elif self.status=='win':
			for s in self.sprites:
				s.move([False,False,False,True], MAXSPEED+1)		
			if self.timer>=2*FPS:
				if self.points>=8:
					if self.level==3:
						self.manager.go_to(Credits())
					else:
						self.manager.go_to(Map(self.level+1))
				else:
					print 'You need 8 bits to pass the level'
					self.manager.go_to(Map(self.level))
	
		elif self.status=='lose':
			self.speed/=2
			self.fox.art.set('dead')
			if self.timer>=2*FPS:
				self.manager.go_to(Map(self.level))
	
	def	check_deaths(self):
		for s in self.sprites:
			if s.delayed_death>0:
				s.collision_action='none'
				s.delayed_death-=1
				if s.delayed_death==3*FPS:
					s.art.set('death')
				elif s.delayed_death==1*FPS:			
					self.remobj(s)
	
	def	generate_stuff(self, i, *args):
		obj_list=[objects.Trap(*args),objects.Hurdle(*args),objects.Chicken(*args)]
		#self.stuff.append(self.addobj(obj_list[i]))
		self.addobj(obj_list[i])
		
	def update_ingame(self):
		#print self.speed,self.scroll_total,self.scroll_total%CELLSIZE
		if self.scroll_total>0 and self.speed>0 and (self.scroll_total%CELLSIZE)<7 and random.random()>0.8:
			#self.stuff.append(self.addobj(objects.Trap([self.dirkeys[2]*(0)*CELLSIZE + self.dirkeys[3]*CELL_HNUM*CELLSIZE, CELLSIZE*random.randrange(2,CELL_VNUM-2)])))
			self.generate_stuff(random.randrange(2),[self.dirkeys[2]*(0)*CELLSIZE + self.dirkeys[3]*(CELL_HNUM+1)*CELLSIZE, CELLSIZE*random.randrange(2,CELL_VNUM-2)])
		if self.scroll_total>0 and self.speed>0 and (self.scroll_total%CELLSIZE)<7 and random.random()>0.8:
			self.generate_stuff(2,[self.dirkeys[2]*(0)*CELLSIZE + self.dirkeys[3]*(CELL_HNUM+1)*CELLSIZE, CELLSIZE*random.randrange(2,CELL_VNUM-2)])	
		
		#end of level
		if self.scroll_total>10*(self.level+1)*CELLSIZE*CELL_VNUM:
			self.timer=0
			self.status='win'
			[self.addobj(objects.Tree([(CELL_VNUM+random.randrange(3,20))*CELLSIZE,random.randrange(CELL_VNUM)*CELLSIZE])) for i in range(50)]		
		
		
		#add scrolling speed while running, unless you change direction
		if True in self.dirkeys:
			self.speed=min(self.maxspeed,self.speed+.1)
			if self.dirkeys==self.lastkeys:
				self.speed=min(self.maxspeed,self.speed+.1)
			else:
				self.lastkeys=self.dirkeys[:]
				if self.speed>1:
					self.speed=max(0,self.speed-.6)
		else:
			self.speed=max(0,self.speed-.1)
	
		#if level scrolling speed is<>0
		if self.speed<>0:	
			#1)
			#check collision with any blocking objects.
			#if check returns 0, the level scrolling speed is set to 0 and no more objects are checked		
			for s in self.sprites:
				action=s.check(self.dirkeys, math.ceil(self.speed), self.fox.art.jumping)
				if action<>'none': print action
				if action=='stop':
					self.speed=0
					break
				elif action=='slow':
					self.speed/=2
					break
				elif action=='win':
					self.status='win'
					self.timer=0
					break
				elif action=='caught':
					self.status='lose'
					self.timer=0
					break
				elif action=='+1 point':
					self.points+=1
					print 'You bit a chicken. You have',self.points,'points now'		
					
		#if level scrolling speed is still<>0
		if self.speed<>0:
			#scroll all objects	
			for s in self.sprites:
				s.move(self.dirkeys, math.ceil(self.speed))
			
			#if screen is scrolling more than one cell
			if self.scroll_index+self.speed>=CELLSIZE:	
				# 1) delete objects out of 5x5 screen size rect
				for s in self.sprites:
					if s.rect.right<-CELL_HNUM*CELLSIZE or s.rect.left>2*CELL_HNUM*CELLSIZE or s.rect.bottom<-CELL_VNUM*CELLSIZE or s.rect.top>2*CELL_VNUM*CELLSIZE:
						if s.rect.right<-CELL_HNUM*CELLSIZE and s.__class__.__name__=='Chicken' and s.collision_action=='+1 point':
							self.points-=1
							print 'You missed a chicken. You have',self.points,'points now'							
						self.remobj(s)				

				# 2) add new objects on the opposite side
				#(KUP,KDOWN,KLEFT,KRIGHT)
				if self.direction[0] in ('l','r') and True in self.dirkeys[2:]:# and True not in self.dirkeys[:2]:
					[self.addobj(objects.Bush([self.dirkeys[2]*(0)*CELLSIZE + self.dirkeys[3]*CELL_HNUM*CELLSIZE, line*CELLSIZE])) for line in [1,CELL_VNUM-1]]
				if self.direction[0] in ('u','d') and True in self.dirkeys[:2]:# and True not in self.dirkeys[:2]:
					[self.addobj(objects.Bush([col*CELLSIZE, self.dirkeys[2]*(0)*CELLSIZE + self.dirkeys[1]*CELL_VNUM*CELLSIZE])) for col in [1,CELL_HNUM-1]]
					


			#increase map and scrolling index
			self.scroll_index=(self.scroll_index+self.speed)%CELLSIZE
			self.scroll_total+=self.speed

			

	def handle_events(self,events):
		for e in events:
			if e.type == pygame.KEYDOWN:
				#random.choice(SOUNDS).play()
				if e.key==pygame.K_ESCAPE:
					sys.exit('ESC key pressed')	
			
			#in game controls
			if self.status=='ingame':
				if e.type in (pygame.KEYDOWN,pygame.KEYUP):
					if e.key in (KUP,KDOWN,KLEFT,KRIGHT):
						self.dirkeys[(KUP,KDOWN,KLEFT,KRIGHT).index(e.key)]=(e.type==pygame.KEYDOWN)
					if True in self.dirkeys and e.key==KJUMP:
						self.fox.jump(e.type==pygame.KEYDOWN)
					
					
					
	
