import art
from constants import *
import random, math


class Dirty_obj(pygame.sprite.DirtySprite): 
	def __init__(self, image, position, size=None):
		pygame.sprite.DirtySprite.__init__(self)
		self.image=image 
		self.rect=self.image.get_rect()
		self.rect.center=position
		
		self.delayed_death=0
		

class Fox(Dirty_obj):
	def __init__(self, position):
		image_name='obj_fox.png'
		self.size=[CELLSIZE,CELLSIZE]
		self.art=art.Art(image_name,self.size)

		self.image=self.art.next()
		self.animationChange=FPS/FRAMES_PER_SECOND
		self.animationCounter=0
		Dirty_obj.__init__(self, self.image, position, self.size)
		print self.rect
		
	def update(self):
		self.animationCounter+=1
		if self.animationCounter>=self.animationChange:
			self.animationCounter=0
			self.image=self.art.next()
			self.dirty=1
			
	def jump(self,jump):
		self.art.jump(jump)
	
	def check(self, dirkeys, speed, jumping):
		return 'none'
	
	def move(self, dirkeys, speed):
		dup,ddown,dleft,dright=dirkeys
		self.animationChange=FPS/(FRAMES_PER_SECOND+speed)
		try:
			self.art.set(('ru','lu','rd','ld','r','l','u','d')[(dright*dup,dleft*dup,dright*ddown,dleft*ddown,dright,dleft,dup,ddown).index(True)])
		except ValueError:
			self.art.stop()
		self.dirty=1
		return True

		

class Farm(Dirty_obj):
	def __init__(self, position):
		self.image_name='map_farm.png'
		self.size=2*[2*CELLSIZE]
		self.art=art.Art(self.image_name,self.size)
		self.image=self.art.next()
		Dirty_obj.__init__(self, self.image, position, self.size)


class Map_Fox(Dirty_obj):
	def __init__(self, position):
		self.image_name='map_fox.png'
		self.size=2*[CELLSIZE]
		self.art=art.Art(self.image_name,self.size)
		self.image=self.art.next()
		self.start_pos,self.end_pos=position
		self.ready=False
		Dirty_obj.__init__(self, self.image, self.start_pos, self.size)
		
		
	def update(self):
		if math.hypot(self.rect.centerx-self.end_pos[0], self.rect.centery-self.end_pos[1])>CELLSIZE:
			self.rect.centerx+=int(math.copysign(2,self.end_pos[0]-self.rect.centerx))
			self.rect.centery+=int(math.copysign(2,self.end_pos[1]-self.rect.centery))
			self.dirty=1
		else:
			self.ready=True
		
	
class Things(Dirty_obj):
	def __init__(self, image_name, position, size):
		self.size=size
		self.art=art.Art(image_name,self.size)
		self.image=self.art.next()
		self.speed=0
		self.art.set('default')
		self.animationChange=FRAMES_PER_SECOND
		self.animationCounter=0
		self.collision_action='none'
		Dirty_obj.__init__(self, self.image, position, self.size)
		


	def check(self, dirkeys, speed, jumping):
		dup,ddown,dleft,dright=dirkeys
		try_centerx=self.rect.centerx-speed*(-1*dleft+1*dright)
		try_centery=self.rect.centery-speed*(-1*dup+1*ddown)
		
		if jumping:
			return 'none'
		if try_centerx>FOX_POS[0]-CELLSIZE/2 and try_centerx<FOX_POS[0]+CELLSIZE/2 and try_centery>FOX_POS[1]-CELLSIZE/2 and try_centery<FOX_POS[1]+CELLSIZE/2:
			return_value=self.collision_action
			self.at_collision()
			return return_value
		return 'none'

	
	def move(self, dirkeys, speed):
		dup,ddown,dleft,dright=dirkeys
		self.rect.centerx-=speed*(-1*dleft+1*dright)
		self.rect.centery-=speed*(-1*dup+1*ddown)
		self.dirty=1		
		self.move_custom()

	def move_custom(self):
		pass

	def update(self):
		self.speed=max(min(self.maxspeed,self.speed+0.1),self.speed-0.1)
		#if self.__class__.__name__=='Horse': print self.speed
		self.rect.centerx+=self.speed
		self.animationCounter+=1
		if self.animationCounter>=self.animationChange:
			self.animationCounter=0
			self.image=self.art.next()
		self.dirty=1				
		
	def at_collision(self):
		pass



class Dog(Things):
	def __init__(self, position):
		image_name=random.choice(['obj_dog_brown.png','obj_dog_grey.png','obj_dog_dalmatian.png'])
		size=[CELLSIZE,CELLSIZE]
		Things.__init__(self, image_name, position, size)
		self.maxspeed=MAXSPEED-1
		self.sprint=0
		self.collision_action='caught'		
		self.change_lane=0

	def move_custom(self):
		if self.change_lane==0:
			if self.rect.centery<2*CELLSIZE:
				self.change_lane=1
			elif self.rect.centery>(CELL_VNUM-2)*CELLSIZE:
				self.change_lane=-1
			elif random.random()>0.7:
				if random.random()>0.7:
					#wrong direction
					self.change_lane=-math.copysign(1,FOX_POS[1]-self.rect.centery)
				else:
					#correct direction
					self.change_lane=math.copysign(1,FOX_POS[1]-self.rect.centery)
			else:
				self.maxspeed=MAXSPEED+random.randrange(-1,2)
		else:
			self.rect.centery+=self.change_lane
			if self.rect.centery%CELLSIZE==0:
				self.change_lane=0
		'''
		self.sprint+=1
		if self.sprint>1*FPS:
			self.sprint=0
			self.maxspeed=MAXSPEED+random.random()
		'''

class Horse(Things):
	def __init__(self, position):
		image_name=random.choice(['obj_horse_blue.png','obj_horse_brown.png','obj_horse_green.png','obj_horse_red.png'])
		size=[2*CELLSIZE,2*CELLSIZE]
		Things.__init__(self, image_name, position, size)	
		self.maxspeed=MAXSPEED-1
		self.sprint=0			
		self.change_lane=0
		self.collision_action='caught'
	'''
	def move_custom(self):
		self.sprint+=1
		if self.maxspeed<10:
			if self.sprint>3*FPS*random.random():
				self.sprint=0
				self.maxspeed=10
		else:
			if self.sprint>3*FPS*random.random():
				self.sprint=0
				self.maxspeed=MAXSPEED-2
	'''			
	def move_custom(self):
		if self.change_lane==0:
			if self.rect.centery<2*CELLSIZE:
				self.change_lane=2
			elif self.rect.centery>(CELL_VNUM-2)*CELLSIZE:
				self.change_lane=-2
			elif random.random()>0.7:
				if random.random()>0.5:
					#wrong direction
					self.change_lane=-math.copysign(1,FOX_POS[1]-self.rect.centery)
				else:
					#correct direction
					self.change_lane=math.copysign(1,FOX_POS[1]-self.rect.centery)
			else:
				self.sprint=2.5*FPS
				self.maxspeed=MAXSPEED+random.randrange(1,3)
				
		elif self.sprint<=0:
			self.maxspeed=MAXSPEED-random.randrange(1,3)
			self.rect.centery+=self.change_lane
			if self.rect.centery%CELLSIZE==0:
				self.change_lane=0
		else:
			self.sprint-=1

			
		
		
class Tree(Things):
	def __init__(self, position):
		image_name='obj_tree.png'
		size=2*[2*CELLSIZE]
		Things.__init__(self, image_name, position, size)
		self.art.set(str(random.randrange(1,10)))
		self.image=self.art.next()		
		self.maxspeed=0
		self.sprint=0			
		self.collision_action='win'

		
class Bush(Things):
	def __init__(self, position):
		image_name='obj_bush.png'
		size=2*[CELLSIZE]
		Things.__init__(self, image_name, position, size)
		self.maxspeed=0
		self.sprint=0			
		self.collision_action='stop'

		
class Trap(Things):	
	def __init__(self, position):
		image_name='obj_trap.png'
		size=[CELLSIZE/2,CELLSIZE/2]
		Things.__init__(self, image_name, position, size)	
		self.maxspeed=0
		self.sprint=0
		self.collision_action='slow'

		
	def at_collision(self):
		self.art.set('hit')	
		self.image=self.art.next()
		self.collision_action='none'
		self.delayed_death=5*FPS

		
class Hurdle(Things):	
	def __init__(self, position):
		image_name='obj_hurdle.png'
		size=[CELLSIZE*4,CELLSIZE*4]
		Things.__init__(self, image_name, position, size)	
		self.maxspeed=0
		self.sprint=0
		self.collision_action='stop'

				
class Chicken(Things):	
	def __init__(self, position):
		image_name='obj_chicken.png'
		size=[CELLSIZE,CELLSIZE]
		Things.__init__(self, image_name, position, size)	
		self.art.set(random.choice(['r','l','d','u','ru','lu','ld','rd']))
		self.maxspeed=1
		self.collision_action='+1 point'
		
	def move_custom(self):
		if self.collision_action=='point' and random.random()>0.8:
			self.art.set(random.choice(['r','l','d','u','ru','lu','ld','rd']))
				
	def at_collision(self):
		self.art.set('death')
		self.collision_action='none'
		self.delayed_death=FPS			
			
		
		
